# simplistic example of parsing through PostgreSQL analyze verbose output
import sys, re

def printHuman(filename, lineNum, parent, operation, estCost, estRows, actualTime, actRows):
	print "%-64s %3s %3s %-20s %10.2f %12.3f %10.3f %12s %12s" \
	% (\
		filename,\
		lineNum,\
		parent,\
		operation,\
		float(estCost),\
		float(actualTime),\
		1000.0 * float(actualTime) / float(estCost),\
		estRows,\
		actRows\
	)

# A sample line for this pattern to match:
#                ->  Bitmap Index Scan on "DbTranRepository_timestamp"  (cost=0.00..448.46 rows=36054 width=0) (actual time=232.683..232.683 rows=39690 loops=1)
pattern = re.compile('\A\ *(?:\-\>)? *(.*?) (?:(?:using |on ).*)?\
 *\(cost\=(\d+\.\d+)\.\.(\d+\.\d+) rows\=(\d+) width\=(\d+)\)\
 \(actual time\=(\d+\.\d+)\.\.(\d+\.\d+) rows\=(\d+) loops\=(\d+)\)')

for inputfilename in sys.argv[1:]:
	# Get the plan file into a list for easy handling.
	inputfile = open(inputfilename, 'r')
	inputlines = inputfile.readlines()
	inputfile.close()
	# Initialize variables used for parent tracking.
	stack = (0,)
	parent = -1
	# Loop through lines from plan, picking out lines with cost/timing info.
	for indx in range(len(inputlines)):
		line = inputlines[indx]
		m = pattern.match(line)
		if m:
			# Get the values into named variables for clarity.
			operation = m.group(1)
			estCostStart = m.group(2)
			estCostFinish = m.group(3)
			estRows = m.group(4)
			width = m.group(5)
			actualTimeStart = m.group(6)
			actualTimeFinish = m.group(7)
			actualRows = m.group(8)
			loops = m.group(9)
			# Keep track of parent using tuple as a stack.
			# This will allow "bill of materials" analysis of plan.
			indent = line.find('->')
			if indent < 0:
				parent = -1
				stack = (indx, )
			else:
				stackLoc = (indent + 3) / 6
				parent = stack[stackLoc - 1]
				stack = stack[:stackLoc] + (indx,)
				line = line[indent + 4:]
			# This is where we will save to the database.
			# For simple test, just show some important data on console.
			printHuman(inputfilename, indx, parent, operation, estCostFinish, estRows, actualTimeFinish, actualRows)

